"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = __importDefault(require("os"));
const convict_1 = __importStar(require("convict"));
const utils_1 = require("@terascope/utils");
// @ts-expect-error no types
const convict_format_with_validator_1 = __importDefault(require("convict-format-with-validator"));
// @ts-expect-error no types
const convict_format_with_moment_1 = __importDefault(require("convict-format-with-moment"));
const connector_utils_1 = require("./connector-utils");
const schema_1 = require("./schema");
(0, convict_1.addFormats)(convict_format_with_validator_1.default);
(0, convict_1.addFormats)(convict_format_with_moment_1.default);
function validateConfig(cluster, schema, namespaceConfig) {
    try {
        const config = (0, convict_1.default)(schema || {});
        config.load(namespaceConfig);
        if (cluster.isMaster) {
            config.validate({
                // IMPORTANT: changing this will break things
                // FIXME
                // false is deprecated and will be removed in ^5.0.0
                // must be warn or strict
                allowed: true,
            });
        }
        return config.getProperties();
    }
    catch (err) {
        throw new utils_1.TSError(err, { reason: 'Error validating configuration' });
    }
}
function extractSchema(fn, sysconfig) {
    if ((0, utils_1.isFunction)(fn)) {
        return fn(sysconfig);
    }
    if ((0, utils_1.isPlainObject)(fn)) {
        return fn;
    }
    return {};
}
/**
 * @param cluster the nodejs cluster metadata
 * @param config the config object passed to the library terafoundation
 * @param sysconfig unvalidated sysconfig
*/
function validateConfigs(cluster, config, sysconfig) {
    if (!(0, utils_1.isPlainObject)(config) || (0, utils_1.isEmpty)(config)) {
        throw new Error('Terafoundation requires a valid application configuration');
    }
    if (!(0, utils_1.isPlainObject)(sysconfig) || (0, utils_1.isEmpty)(sysconfig)) {
        throw new Error('Terafoundation requires a valid system configuration');
    }
    const schema = extractSchema(config.config_schema, sysconfig);
    schema.terafoundation = (0, schema_1.foundationSchema)(sysconfig);
    const result = {};
    if (config.schema_formats) {
        config.schema_formats.forEach((format) => {
            convict_1.default.addFormat(format);
        });
    }
    const schemaKeys = (0, utils_1.concat)(Object.keys(schema), Object.keys(sysconfig));
    for (const schemaKey of schemaKeys) {
        const subSchema = schema[schemaKey] || {};
        const subConfig = sysconfig[schemaKey] || {};
        result[schemaKey] = validateConfig(cluster, subSchema, subConfig);
        if (schemaKey === 'terafoundation') {
            result[schemaKey].connectors = {};
            const connectors = subConfig.connectors || {};
            for (const [connector, connectorConfig] of Object.entries(connectors)) {
                const connectorSchema = (0, connector_utils_1.getConnectorSchema)(connector);
                result[schemaKey].connectors[connector] = {};
                for (const [connection, connectionConfig] of Object.entries(connectorConfig)) {
                    result[schemaKey].connectors[connector][connection] = validateConfig(cluster, connectorSchema, connectionConfig);
                }
            }
        }
    }
    // Annotate the config with some information about this instance.
    const hostname = os_1.default.hostname();
    if (process.env.POD_IP) {
        result._nodeName = process.env.POD_IP;
    }
    else if (cluster.worker) {
        result._nodeName = `${hostname}.${cluster.worker.id}`;
    }
    else {
        result._nodeName = hostname;
    }
    return result;
}
exports.default = validateConfigs;
//# sourceMappingURL=validate-configs.js.map